import atexit
import datetime
from datetime import date

import flask
from pythonping import ping
from flask import Flask, jsonify, request
from flask_migrate import Migrate
from flask_sqlalchemy import SQLAlchemy
from apscheduler.schedulers.background import BackgroundScheduler
from sqlalchemy import desc, column

from config import *

app = Flask(__name__)
app.config['SQLALCHEMY_DATABASE_URI'] = DB_URL
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = DB_TRACK_NODIFICATION
db = SQLAlchemy(app)
migrate = Migrate(app, db)


class Camera(db.Model):
    __tablename__ = 'camera'

    id = db.Column(db.Integer, primary_key=True)
    camera_ip = db.Column(db.String())

    def __init__(self, camera_ip):
        self.camera_ip = camera_ip

    def __repr__(self):
        return '<ip {}>'.format(self.camera_ip)

    @property
    def serialize(self):
        return {
            'ip': self.camera_ip
        }


def dump_datetime(value):
    """Deserialize datetime object into string form for JSON processing."""
    if value is None:
        return None
    return [value.strftime("%Y-%m-%d"), value.strftime("%H:%M:%S")]


class Stat(db.Model):
    __tablename__ = 'stat'

    id = db.Column(db.Integer, primary_key=True)
    camera_ip = db.Column(db.String())
    time = db.Column(db.Float)
    date = db.Column(db.DateTime, default=db.func.now())

    def __init__(self, camera_ip, time):
        self.camera_ip = camera_ip
        self.time = time

    def __repr__(self):
        return '<id {}>'.format(self.id)

    @property
    def serialize(self):
        return {
            'ip': self.camera_ip,
            'time': self.time,
            'date': dump_datetime(self.date)
        }


time = 60


@app.route('/api/v1/statistic', methods=['GET'])
def get_statistic():
    start_date = request.args.get('start_date', datetime.datetime.now() - datetime.timedelta(days=30))
    statistic = Stat.query.filter(Stat.date >= start_date).all()
    return jsonify({'data': [i.serialize for i in statistic]})


@app.route('/api/v1/time', methods=['GET'])
def get_interval():
    return jsonify({'time': time, 'unit_of_time': 'sec'})


@app.route('/api/v1/camera', methods=['POST'])
def post_ip():
    ips = request.json
    for ip in ips["ips"]:
        camera = Camera(ip)
        db.session.add(camera)
        db.session.commit()
    status_code = flask.Response(status=200)
    return status_code


@app.route('/api/v1/time', methods=['POST'])
def post_time():
    new_time = request.args.get('time')
    global time
    time = new_time
    scheduler.reschedule_job('scheduler', trigger='interval', seconds=int(new_time))
    status_code = flask.Response(status=200)
    return status_code


@app.route('/api/v1/camera', methods=['DELETE'])
def remove_camera():
    ips = request.json
    for ip in ips["ips"]:
        Camera.query.filter(Camera.camera_ip == ip).delete()
    db.session.commit()
    status_code = flask.Response(status=200)
    return status_code


@app.route('/api/v1/scheduler/stop', methods=['POST'])
def stop_scheduler():
    scheduler.pause()
    status_code = flask.Response(status=200)
    return status_code


@app.route('/api/v1/scheduler/start', methods=['POST'])
def start_scheduler():
    scheduler.resume()
    status_code = flask.Response(status=200)
    return status_code


@app.route('/api/v1/db/auto-clean-time', methods=['POST'])
def update_auto_clean_db():
    scheduler.resume()
    status_code = flask.Response(status=200)
    return status_code


def ip_ping():
    for camera in Camera.query.all():
        response_list = ping(camera.camera_ip, size=40, count=3)
        av_time = response_list.rtt_avg_ms
        stat = Stat(camera.camera_ip, av_time)
        db.session.add(stat)
        db.session.commit()


def auto_clean_db():
    clean_date = datetime.datetime.now() - DB_AUTOCLEAN_INTERVAL
    old_stats = Stat.query.filter(Stat.date < clean_date).all()
    for old_stat in old_stats:
        db.session.delete(old_stat)
    db.session.commit()


scheduler = BackgroundScheduler()
scheduler.add_job(id="scheduler", func=ip_ping, trigger="interval", seconds=time)
scheduler.start()

# Shut down the scheduler when exiting the app
atexit.register(lambda: scheduler.shutdown())

scheduler2 = BackgroundScheduler()
scheduler2.add_job(id="dbclean", func=auto_clean_db, trigger="interval", seconds=DB_AUTOCLEAN_INTERVAL.total_seconds())
scheduler2.start()

# Shut down the scheduler when exiting the app
atexit.register(lambda: scheduler2.shutdown())


if __name__ == '__main__':
    app.run(debug=True)
    app.run(use_reloader=False)
